﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractConditionTest
    {
        [Test]
        public void TestConditionAndConditionChanged()
        {
            ConcreteCondition target = new ConcreteCondition(false);
            Assert.IsFalse(target.Condition);

            bool conditionChanged = false;
            target.ConditionChanged += delegate(object sender, EventArgs e)
                {
                    Assert.IsFalse(conditionChanged);
                    conditionChanged = true;
                };

            target.SetCondition(true);
            Assert.IsTrue(target.Condition);
            Assert.IsTrue(conditionChanged);
        }

        [Test]
        public void TestParameterlessConstructor()
        {
            var target = new ConcreteCondition();
            Assert.IsFalse(target.Condition);
        }

        [Test]
        public void TestInitializingToTrue()
        {
            var target = new ConcreteCondition(true);
            Assert.IsTrue(target.Condition);
        }

        [Test]
        public void TestToggleCondition()
        {
            var target = new ConcreteCondition();
            Assert.IsFalse(target.Condition);
            target.ToggleCondition();
            Assert.IsTrue(target.Condition);
            target.ToggleCondition();
            Assert.IsFalse(target.Condition);
        }
    }
}
