﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;
using System.Windows;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractControlTest
    {

        #region " ToolTip "
        [Test]
        public void TestToolTip()
        {
            //test default value
            ConcreteControl target = new ConcreteControl();
            Assert.IsNull(target.ToolTip);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "ToolTip")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetToolTip(test);
            Assert.AreEqual(test, target.ToolTip);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        public void TestToolTipNewline()
        {
            ConcreteControl target = new ConcreteControl();

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "ToolTip")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = @"line 1\" + "nline 2";
            target.SetToolTip(test);
            Assert.AreEqual("line 1" + Environment.NewLine + "line 2", target.ToolTip);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        public void Setting_ToolTip_to_empty_string_returns_null()
        {
            ConcreteControl target = new ConcreteControl();
            target.SetToolTip("some value");
            Assert.IsNotNull(target.ToolTip);
            target.SetToolTip(string.Empty);
            Assert.IsNull(target.ToolTip);
        }
        #endregion

        #region " Visible and VisibleCondition "
        [Test]
        public void TestVisibleAndVisibleCondition()
        {
            //test default value
            ConcreteControl target = new ConcreteControl();
            Assert.IsTrue(target.Visible);
            Assert.IsNull(target.VisibleCondition);

            //test setting value and property changed event
            bool propertyChanged1 = false;
            bool propertyChanged2 = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Visible")
                {
                    Assert.IsFalse(propertyChanged1);
                    propertyChanged1 = true;
                }
                else if (e.PropertyName == "VisibleCondition")
                {
                    Assert.IsFalse(propertyChanged2);
                    propertyChanged2 = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            ConcreteCondition newCondition = new ConcreteCondition(false);
            target.VisibleCondition=  newCondition;
            Assert.IsFalse(target.Visible);
            Assert.IsTrue(propertyChanged1);
            Assert.IsTrue(propertyChanged2);

            propertyChanged1 = false;
            propertyChanged2 = false;

            newCondition.SetCondition(true);
            Assert.IsTrue(target.Visible);
            Assert.IsTrue(propertyChanged1);
            Assert.IsFalse(propertyChanged2);
        }
        #endregion

        #region " Padding "
        [Test]
        public void TestPadding()
        {
            //test default value
            ConcreteControl target = new ConcreteControl();
            Assert.AreEqual(new Thickness(0), target.Padding);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Padding")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            Thickness test = new Thickness(1, 2, 3, 4);
            target.SetPadding(test);
            Assert.AreEqual(test, target.Padding);
            Assert.IsTrue(propertyChanged);
        }
        #endregion

        #region " Margin "
        [Test]
        public void TestMargin()
        {
            //test default value
            ConcreteControl target = new ConcreteControl();
            Assert.AreEqual(new Thickness(0), target.Margin);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Margin")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            Thickness test = new Thickness(1, 2, 3, 4);
            target.SetMargin(test);
            Assert.AreEqual(test, target.Margin);
            Assert.IsTrue(propertyChanged);
        }
        #endregion

        #region " Width "
        [Test]
        public void TestWidth()
        {
            //test default value
            var target = new ConcreteControl();
            Assert.AreEqual(double.NaN, target.Width);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Width")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            double test = 15.2;
            target.SetWidth(test);
            Assert.AreEqual(test, target.Width);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestWidthNegative()
        {
            var target = new ConcreteControl();
            double test = -0.01;
            target.SetWidth(test);
        }
        #endregion

        #region " Height "
        [Test]
        public void TestHeight()
        {
            //test default value
            var target = new ConcreteControl();
            Assert.AreEqual(double.NaN, target.Height);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Height")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            double test = Math.PI;
            target.SetHeight(test);
            Assert.AreEqual(test, target.Height);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestHeightNegative()
        {
            var target = new ConcreteControl();
            double test = -0.01;
            target.SetHeight(test);
        }
        #endregion

        [Test]
        public void TestSettingLogger()
        {
            var target = new ConcreteControl();
            TestViewModelHelper.SetMockLogger(target);
            Assert.IsNotNull(target.GetLogger());
        }
    }
}
