﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractToolBarTest
    {
        #region " Header "
        [Test]
        public void TestHeader()
        {
            //test default value
            ConcreteToolBar target = new ConcreteToolBar();
            Assert.IsNull(target.Header);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Header")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetHeader(test);
            Assert.AreEqual(test, target.Header);
            Assert.IsTrue(propertyChanged);

            propertyChanged = false;
            target.SetHeader(null);
            Assert.IsNull(target.Header);
            Assert.IsTrue(propertyChanged);

        }
        #endregion

        #region " Name "
        [Test]
        public void TestName()
        {
            //test default value
            ConcreteToolBar target = new ConcreteToolBar();
            Assert.AreEqual(target.ID, target.Name);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Name")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetName(test);
            Assert.AreEqual(test, target.Name);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNameNull()
        {
            //should throw exception on null value
            ConcreteToolBar target = new ConcreteToolBar();
            target.SetName(null);
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestNameEmpty()
        {
            //should throw exception on null value
            ConcreteToolBar target = new ConcreteToolBar();
            target.SetName(string.Empty);
        }
        #endregion

        #region " Items "
        [Test]
        public void TestItems()
        {
            //test default value
            ConcreteToolBar target = new ConcreteToolBar();
            Assert.IsNull(target.Items);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Items")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            Collection<IToolBarItem> i = new Collection<IToolBarItem>();
            target.SetItems(i);
            Assert.AreEqual(i, target.Items);
            Assert.IsTrue(propertyChanged);
        }
        #endregion

    }
}
