﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.Collections.ObjectModel;

namespace SoapBox.Core
{
    [TestFixture]
    public class ExtensionServiceTest
    {

        [Test]
        public void TestSort_EmptyList()
        {
            IExtensionService target = new ExtensionService();

            IEnumerable<IExtension> extensions = new Collection<IExtension>();
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(0, sortedExtensions.Count);
        }

        [Test]
        public void TestSort_OneItem()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension());
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(1, sortedExtensions.Count);
            Assert.AreEqual(extensions[0].ID, sortedExtensions[0].ID);
        }

        [Test]
        public void TestSort_TwoItems_NoOrder()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension());
            extensions.Add(new ConcreteExtension());
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(2, sortedExtensions.Count);
            //should come out in the same order
            Assert.AreEqual(extensions[0].ID, sortedExtensions[0].ID);
            Assert.AreEqual(extensions[1].ID, sortedExtensions[1].ID);
        }

        [Test]
        public void TestSort_ThreeItems_NoOrder()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension());
            extensions.Add(new ConcreteExtension());
            extensions.Add(new ConcreteExtension());
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            //should come out in the same order
            Assert.AreEqual(extensions[0].ID, sortedExtensions[0].ID);
            Assert.AreEqual(extensions[1].ID, sortedExtensions[1].ID);
            Assert.AreEqual(extensions[2].ID, sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertBefore_321()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", null, RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("2", "1", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("3", "2", RelativeDirection.Before));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("3", sortedExtensions[0].ID);
            Assert.AreEqual("2", sortedExtensions[1].ID);
            Assert.AreEqual("1", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertBefore_231()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", null, RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("2", "3", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("3", "1", RelativeDirection.Before));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("2", sortedExtensions[0].ID);
            Assert.AreEqual("3", sortedExtensions[1].ID);
            Assert.AreEqual("1", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertBefore_132()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "3", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("2", null, RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("3", "2", RelativeDirection.Before));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("1", sortedExtensions[0].ID);
            Assert.AreEqual("3", sortedExtensions[1].ID);
            Assert.AreEqual("2", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertBefore_213()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "3", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("2", "1", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("3", null, RelativeDirection.Before));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("2", sortedExtensions[0].ID);
            Assert.AreEqual("1", sortedExtensions[1].ID);
            Assert.AreEqual("3", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertBefore_312()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "2", RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("2", null, RelativeDirection.Before));
            extensions.Add(new ConcreteExtension("3", "1", RelativeDirection.Before));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("3", sortedExtensions[0].ID);
            Assert.AreEqual("1", sortedExtensions[1].ID);
            Assert.AreEqual("2", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertAfter_321()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "2", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("2", "3", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("3", null, RelativeDirection.After));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("3", sortedExtensions[0].ID);
            Assert.AreEqual("2", sortedExtensions[1].ID);
            Assert.AreEqual("1", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertAfter_231()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "3", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("2", null, RelativeDirection.After));
            extensions.Add(new ConcreteExtension("3", "2", RelativeDirection.After));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("2", sortedExtensions[0].ID);
            Assert.AreEqual("3", sortedExtensions[1].ID);
            Assert.AreEqual("1", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertAfter_132()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", null, RelativeDirection.After));
            extensions.Add(new ConcreteExtension("2", "3", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("3", "1", RelativeDirection.After));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("1", sortedExtensions[0].ID);
            Assert.AreEqual("3", sortedExtensions[1].ID);
            Assert.AreEqual("2", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertAfter_213()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "2", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("2", null, RelativeDirection.After));
            extensions.Add(new ConcreteExtension("3", "1", RelativeDirection.After));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("2", sortedExtensions[0].ID);
            Assert.AreEqual("1", sortedExtensions[1].ID);
            Assert.AreEqual("3", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSort_ThreeItems_InsertAfter_312()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions = new Collection<IExtension>();
            extensions.Add(new ConcreteExtension("1", "3", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("2", "1", RelativeDirection.After));
            extensions.Add(new ConcreteExtension("3", null, RelativeDirection.After));
            IList<IExtension> sortedExtensions = target.Sort(extensions);

            Assert.AreEqual(3, sortedExtensions.Count);
            Assert.AreEqual("3", sortedExtensions[0].ID);
            Assert.AreEqual("1", sortedExtensions[1].ID);
            Assert.AreEqual("2", sortedExtensions[2].ID);
        }

        [Test]
        public void TestSortAndJoin()
        {
            IExtensionService target = new ExtensionService();

            Collection<IExtension> extensions1 = new Collection<IExtension>(); // correct order is 312
            extensions1.Add(new ConcreteExtension("1", "3", RelativeDirection.After));
            extensions1.Add(new ConcreteExtension("2", "1", RelativeDirection.After));
            extensions1.Add(new ConcreteExtension("3", null, RelativeDirection.After));

            Collection<IExtension> extensions2 = new Collection<IExtension>(); // correct order is 546
            extensions2.Add(new ConcreteExtension("4", "5", RelativeDirection.After));
            extensions2.Add(new ConcreteExtension("5", null, RelativeDirection.After));
            extensions2.Add(new ConcreteExtension("6", "4", RelativeDirection.After));

            var joinItem = new ConcreteExtension("7", null, RelativeDirection.After);

            IList<IExtension> sortedExtensions = target.SortAndJoin(extensions1, joinItem, extensions2).ToList();

            Assert.AreEqual(7, sortedExtensions.Count);
            Assert.AreEqual("3", sortedExtensions[0].ID);
            Assert.AreEqual("1", sortedExtensions[1].ID);
            Assert.AreEqual("2", sortedExtensions[2].ID);
            Assert.AreEqual("7", sortedExtensions[3].ID);
            Assert.AreEqual("5", sortedExtensions[4].ID);
            Assert.AreEqual("4", sortedExtensions[5].ID);
            Assert.AreEqual("6", sortedExtensions[6].ID);
        }
    }
}
